# -----------------------------------------------------------------------------
sub oraBkup
# -----------------------------------------------------------------------------
{
  ($thishost) = @_;

  if ($up[$thishost] ne "Y")
  {
    &logit("(Down/Inhibited)",$thishost);
    return 1; 
  }

  if ($asmflag[$thishost])
  {
    &logit("(This check not applicable to +ASM instance)",$thishost);
    return 1;
  }
    
  $this_age = $mr_age[$thishost];
  $this_age = 99999998 if $this_age eq "";
  $bckhours[$thishost] = $this_age;
  if ($bckhours[$thishost] =~ "9999999") 
  {
    $bckhours[$thishost] = "(Never)";
  }

  # Return if they don't want age checking for this instance
  if ($backup_age[$thishost] eq "" || $backup_age[$thishost] eq "N")
  {
    logit("Backup Aging not requested for this instance",$thishost);
    return 0;
  }

  #---- Backup_Age "BIAS" logic - START
  $ba_bias_file = "/opt/dbamon/adm/ba_bias_Oracle.txt"; 
  if (-e "$ba_bias_file") 
  {
    &logit("oraBkup.pl | Found ba_bias_file=$ba_bias_file",$thishost);
    chop($ba_bias = `cat $ba_bias_file | grep -v '^#' | head -1`); 
    if ($ba_bias > 0)
    {
      $backup_age[$thishost] = $backup_age[$thishost] * $ba_bias;
      &logit("oraBkup.pl | Backup bias value ba_bias=$ba_bias is valid - Updated backup_age[$thishost]=$backup_age[$thishost]",$thishost);
    }
  }
  #---- Backup_Age "BIAS" logic - END

  $backup_age_lvl0[$thishost] = $backup_age[$thishost] * 7;
  $backup_age_lvl0[$thishost] = (15*24) if $backup_age_lvl0[$thishost] > (15*24);  # If > 15 days

  $t_lvl0_days = int($backup_age_lvl0[$thishost] / 24);

  $running_str_lvl0 = "";
  $running_str = "";

  #
  # bb 11Oct99 - Check to see if there is a backup age inhibit 
  # entry.
  #
  if ($use_repository eq "Y")
  {
    $rc = &baICheck($thishost);
    if ($rc > 0)
    {
      $backup_method_long[$thishost] = "bkInhibit";
      $bainhibit[$thishost] = 1;
      $bckhours_c[$thishost] = "black";
      $bckhours[$thishost]   = "(I)";
      $msg = sprintf("Backup Age Checking Inhibited - Start: %s End: %s Created: %s User: %s Reason: %s",$bai_sts,$bai_ets,$bai_ts,$bai_user,$bai_reason);
      &logit($msg,$thishost);
      return 0;
    }
  }
  else
  {
    logit("Backup Aging Inhibit not possible because Use_Oracle != Y",$thishost);
  }

  if ($backup_method_long[$thishost] eq "bkNone" && $id1[$thishost] eq "")
  {
    $bckhours_c[$thishost] = "yellow";
    &event("W",$thishost,"Backup Method for this DB is $backup_method_long[$thishost]","BkpNone","DBA516W");
    return 0;
  }

  if ($backup_method_long[$thishost] ne "bkRMAN" && 
      $backup_method_long[$thishost] ne "bkEXP" &&
      $backup_method_long[$thishost] ne "bkTBS" &&
      $backup_method_long[$thishost] ne "bkBCV" &&
      $backup_method_long[$thishost] ne "bkFULL")
  {
    $msg  = "Since Backup_Age: is on for this DB, you should be\n";
    $msg .= "running RMAN backups. Establish RMAN,EXP,FULL\n";
    $msg .= "backups and run a LVL0 backup to satisfy this check.\n";
    $msg .= "\nDiagnostic Data:\n";
    $msg .= $oraBkupStack[$thishost];

    system("rm $tmpfl6 > /dev/null 2>&1");
    open(TMPFL6, ">$tmpfl6") || &error_msg("Can't open output $tmpfl6 - $!");
    print TMPFL6 $msg;
    close(TMPFL6);

    $bckhours_c[$thishost] = "yellow";

    &event("W",$thishost,"Backup Method for this DB is $backup_method_long[$thishost] - Backup_Age: requires that Backup Method be RMAN; EXP; TBS or FULL","BkpMethod","DBA513W",$tmpfl6);
    return 0;
  }

  #running  = "?"

  $bckhours_c[$thishost] = "lime";

  #
  # Check age vs. threshold - backup NOT running
  #

  # Just in case we exceed, set these vars and make report
  $rounded1 = int($this_age / 10);
  $rounded1 = $rounded1 * 10;
  $rounded2 = $rounded1 + 9;

  $msg  = "RMAN Backup Details:\n";
  $msg .= "Hours since last LVL0/LVL1 backup: $this_age\n";
  $msg .= "Threshold for this instance: $backup_age[$thishost]\n";
  $msg .= "Backup Rerun Command for this instance: $backup_command[$thishost]\n";
  $msg .= "Most recent *Any* backup age (hours): $mr_age[$thishost]\n";
  $msg .= "Most recent ARC backup age (hours): $mr_age_arc[$thishost]\n";
  $msg .= "Most recent LVL0 backup age (hours): $mr_age_lvl0[$thishost]\n";
  $msg .= "Most recent LVL1 backup age (hours): $mr_age_lvl1[$thishost]\n";
  $msg .= "Most recent BCV backup age (hours): $mr_age_bcv[$thishost]\n";
  $msg .= "Most recent EXP backup age (hours): $mr_age_exp[$thishost]\n";

  $zdays = int($mr_age_lvl0[$thishost] / 24);

  system("rm $tmpfl6 > /dev/null 2>&1");
  open(TMPFL6, ">$tmpfl6") || &error_msg("Can't open output $tmpfl6 - $!");
  print TMPFL6 $msg;
  close(TMPFL6);

  # Check against threshold
  ##print "this_age=$this_age t=$backup_age[$thishost] \n";

  #$zz = "
  #mr_age_lvl0=$mr_age_lvl0[$thishost] 
  #mr_age_lvl1=$mr_age_lvl1[$thishost] 
  #backup_method_long=$backup_method_long[$thishost] 
  #backup_age_lvl0=$backup_age_lvl0[$thishost] 
  #backup_age=$backup_age[$thishost]";
  #print "$zz";

  if ($this_age > $backup_age[$thishost])
  {
    #Set appropriate backup messagea
    if ($this_age =~ /9999999/) 
    {
      $bckmsg = "Backup Never Run"; 
    }
    else
    {
      $bckmsg = "$rounded1-$rounded2 Hours since last $backup_method_long[$thishost] DB Backup";

      #---- 4.10 06Jun05 BB
      #---- If the age now is >= 2x the threshold, then always make "C" event (ticket)
      if ($this_age >= (2 * $backup_age[$thishost]))
      {
        &event("C",$thishost,"(2X Threshold Exceeded) $bckmsg - Threshold: $backup_age[$thishost]","BkpAge","DBA514",$tmpfl6);
        system("cat $tmpfl6 | dbamon_mailer -s 'DBAmon oraBkup() $titleraw[$thishost] 2X BA Threshold - $this_age / $backup_age[$thishost]' $admin_email") if $debug_514;
      }
    }

    if ($backup_running[$thishost] eq "Y")
    {
      $running_str = "(Backup IS Running Now)";
      $bckhours_c[$thishost] = "yellow";
      &event("U",$thishost,"$bckmsg - threshold: $backup_age[$thishost] $running_str","BkpAge","DBA512",$tmpfl6);
    }
    else
    {
      #
      # 3.23D 08Jul00 BB
      #   we did not issue the restart on the last iteration
      #   (in which case it should be running now), then issue
      #   the restart command.
      #
      ##if ($company eq "thecompany")
      ##{
      ##  chop($datechk = `date +%C%y/%m/%d-%H:%M:%S`);
      ##  if ($datechk ge "2001/05/19-12:00:00" &&  $datechk le "2001/05/21-10:00:00")
      ##  {
      ##    $backup_command[$thishost] = "";
      ##    logit("Backup_Command: Forced to NULL for host: $hostname[$thishost]:$title1[$thishost] ");
      ##  }
      ##}

      if ($backup_command[$thishost] ne "" && $backup_rerun_iter[$thishost] != ($iter_count - 1) )
      {
        # Issue rerun
        $cmd = "$probe[$thishost] bkrun \"$backup_command[$thishost]\"";
        ##print "backup rerun cmd=$cmd \n";
	remsh($cmd,$tmpfl3,$thishost);

        ##print "tmpf3: \n";
	##system("cat $tmpfl3");

        $backup_rerun_iter[$thishost] = $iter_count;

        # warning event
	$running_str = "(Backup Was Rerun)";
	$bckhours_c[$thishost] = "yellow";

        &event("W",$thishost,"$bckmsg - threshold: $backup_age[$thishost] $running_str","BkpAge","DBA511W",$tmpfl6);
      }
      else
      {
	$running_str = "(Backup IS NOT Running Now)";
	$bckhours_c[$thishost] = "hotpink";

        &event("C",$thishost,"$bckmsg - threshold: $backup_age[$thishost] $running_str","BkpAge","DBA511C",$tmpfl6);
      }
    }
  }
  elsif (
         $backup_method_long[$thishost] eq "bkRMAN" &&
         $arclog[$thishost] eq "ArcLog" &&
         $mr_age_lvl0[$thishost] > $backup_age_lvl0[$thishost] &&
         $mr_backup_type[$thishost] ne "OFFLINE" &&
         $standby[$thishost] ne "STANDBY" 
        )
  {
    $zz = "
      mr_age_lvl0=$mr_age_lvl0[$thishost] 
      mr_age_lvl1=$mr_age_lvl1[$thishost] 
      backup_age_lvl0=$backup_age_lvl0[$thishost] 
      backup_age=$backup_age[$thishost]";

    $bckmsg = "$zdays Days since last RMAN LVL0";

    # Temp
    $lvl0_sev = "C";

    if ($backup_running[$thishost] eq "Y")
    {
      $lvl0_sev = "W";
    }

    $running_str_lvl0 = "(Backup IS Running Now)" if $backup_running[$thishost] eq "Y";

    if (
        ($backup_command[$thishost] =~ /LVL0/ || $backup_command[$thishost] =~ /FRADB/) && 
        $backup_running[$thishost] ne "Y" &&
        (
         $backup_rerun_iter_lvl0[$thishost] != ($iter_count - 1) ||
         $iter_count == 1 
        )
       )
    {
      # Issue rerun
      $cmd = "$probe[$thishost] bkrun \"$backup_command[$thishost]\"";
      print "$titleraw[$thishost] - LVL0 backup invoked - cmd=$cmd \n";
      remsh($cmd,$tmpfl3,$thishost);
      ##print "tmpf3: \n";
      ##system("cat $tmpfl3");
      $backup_rerun_iter_lvl0[$thishost] = $iter_count;

      # warning event
      $running_str_lvl0 .= "(Backup Was Rerun)";
      $bckhours_c[$thishost] = "yellow";
      $lvl0_sev = "W";
    }

    &event("$lvl0_sev",$thishost,"$bckmsg - Threshold: $backup_age_lvl0[$thishost] (Hours) $t_lvl0_days (Days) $running_str_lvl0","BkpLVL0Age","DBA517",$tmpfl6);
    ##system("echo '$zz' | dbamon_mailer -m -s 'DBAmon $titleraw[$thishost] DaysSinceLVL0=$zdays T=$t_lvl0_days i=$iter_count $running_str_lvl0' $admin_email");

  }
  else    # Age not exceeded
  {
    $msg = sprintf("OK - Age=%s Threshold=%s",$this_age,$backup_age[$thishost]);
    &logit($msg,$thishost);
  }

  &logDetails($thishost,"</table>");
  &logDetails($thishost,"<h3>Oracle Backup Status</h3>");
  &logDetails($thishost,"<table border=1 bgcolor=antiquewhite>");
  &logDetails($thishost,"<tr bgcolor=slate><th>Parameter<th>Value<th>Comments</tr>");
  &logDetails($thishost,"<tr><td>$f_s <b>Hours since last LVL0/LVL1 backup:</b><td>$f_s $this_age");
  &logDetails($thishost,"<tr><td>$f_s <b>Threshold for this instance (Any Backup):</b><td>$f_s $backup_age[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Threshold for this instance (LVL0 Backup):</b><td>$f_s $backup_age_lvl0[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Backup Rerun Command:</b><td>$f_s $backup_command[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent *Any* backup age (hours):</b><td>$f_s  $mr_age[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent ARC backup age (hours): </b><td>$f_s $mr_age_arc[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent LVL0 backup age (days): </b><td>$f_s $zdays");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent LVL0 backup age (hours): </b><td>$f_s $mr_age_lvl0[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent LVL1 backup age (hours): </b><td>$f_s $mr_age_lvl1[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s <b>Most recent BCV backup age (hours): </b><td>$f_s $mr_age_bcv[$thishost]");
  &logDetails($thishost,"</table><hr>");

  #---- Verify that a backup schedule exists
  &debug($thishost, "bkup_sched_lvl[$thishost]=$bkup_sched_lvl[$thishost]");
  if (
      $bkup_sched_lvl[$thishost] ne "" && 
      $bkup_sched_lvl[$thishost] != 1
     )
  {
    #---- No schedule exists for this DB
    #---- See if a backup age risk acceptance exists and is current for this instance
    eval 'require "risk_acceptance.pl";';

    #foreach (sort keys %backupNo) { print "DEBUG backupNo{$_}=$backupNo{$_}\n"; }
    $key = $titleraw_slash[$thishost];
    chop($today = `date +%Y/%m/%d`);
    &debug($thishost, "backupNo{$key}=$backupNo{$key} today=$today");

    if ($backupNo{$key} && $today lt $backupNo{$key})
    {
      &logit("Backup Schedule not found, but valid RA exists - Backup Type: LVL*",$thishost);
    }
    else
    {
      #---- If we have had a BCV backup within the last 2 weeks, then NO EVENT
      if ($mr_age_bcv[$thishost] ne "" && $mr_age_bcv[$thishost] < (24*14))  
      {
        &logit("This DB has a recent BCV backup, so a LVL backup schedule is not applicable - mr_age_bcv=$mr_age_bcv[$thishost]",$thishost);
      }
      #---- If we have had an OFFLINE backup within the last 2 weeks, then NO EVENT
      elsif ($mr_age_off[$thishost] ne "" && $mr_age_off[$thishost] < (24*14))  
      {
        &logit("This DB has a recent OFFLINE backup, so a LVL backup schedule is not applicable - mr_age_bcv=$mr_age_bcv[$thishost]",$thishost);
      }
      #---- Dont require a backup schedule if this DB runs exports 
      elsif ($mr_age_exp[$thishost] ne "" && $mr_age_exp[$thishost] < (24*14))  
      {
        &logit("This DB has a recent EXP backup, so a LVL backup schedule is not applicable - mr_age_bcv=$mr_age_bcv[$thishost]",$thishost);
      }
      #---- Dont require a backup schedule if this is a standby DB
      elsif ($standby[$thishost] eq "STANDBY")  
      {
        &logit("This DB is a standby, so a LVL backup schedule is not applicable - standby=$standby[$thishost]",$thishost);
      }
      else
      {
        &event("C",$thishost,"Normal Backup Schedule Not Found - Backup Type: LVL*","BkupSchedLVL","DBA380");
      }
    }
  }
}
# -----------------------------------------------------------------------------
1;
